package com.xkrs.utils;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneOffset;
import java.time.format.DateTimeFormatter;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * 日期时间工具
 */
public class DateTimeUtil {

    public final static String COMMON_PATTERN_DATETIME = "yyyy-MM-dd HH:mm:ss";
    public final static String COMMON_PATTERN_DATE = "yyyy-MM-dd";
    public final static DateTimeFormatter COMMON_FORMATTER_DATETIME = DateTimeFormatter.ofPattern(COMMON_PATTERN_DATETIME);
    public final static DateTimeFormatter COMMON_FORMATTER_DATE = DateTimeFormatter.ofPattern(COMMON_PATTERN_DATE);
    public final static ZoneOffset DEFAULT_ZONE_OFFSET = ZoneOffset.of("+8");

    /**
     * LocalDateTime转字符串
     */
    public static String dateTimeToString(LocalDateTime dateTime) {
        assert dateTime != null;
        return COMMON_FORMATTER_DATETIME.format(dateTime);
    }

    /**
     * 字符串转LocalDateTime
     */
    public static LocalDateTime stringToDateTime(String date) {
        LocalDateTime parse = null;
        try {
            parse = LocalDateTime.parse(date, COMMON_FORMATTER_DATETIME);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if (parse == null) {
            return LocalDateTime.parse(date, COMMON_FORMATTER_DATE);
        }
        return parse;
    }

    /**
     * 字符串转LocalDateTime（格式化）
     */
    public static LocalDateTime stringToDateTimeFormatter(String dateStr, DateTimeFormatter formatter) {
        assert dateStr != null;
        return LocalDateTime.parse(dateStr, formatter);
    }


    /**
     * 时间戳转日期
     */
    public static LocalDateTime timeMillisToDate(long timeMillis) {
        Instant instant = Instant.ofEpochMilli(timeMillis);
        return LocalDateTime.ofInstant(instant, DEFAULT_ZONE_OFFSET);
    }

    /**
     * 时间戳转时间再转字符串
     */
    public static String timeMillisToString(long timeMillis) {
        LocalDateTime localDateTime = LocalDateTime.ofEpochSecond(timeMillis, 0, ZoneOffset.ofHours(8));
        return COMMON_FORMATTER_DATETIME.format(localDateTime);
    }

    /**
     * 获取当前时间 yyyy-MM-ss hh:mm:ss
     */
    public static String getNowTimeStr() {
        return COMMON_FORMATTER_DATETIME.format(LocalDateTime.now());
    }

    /**
     * 判断时间格式 格式必须为 "YYYY-MM-DD HH:mm:ss"
     */
    public static boolean isValidDateTime(String sDateTime) {
        final int minLen = 19;
        if ((sDateTime == null) || (sDateTime.length() < minLen)) {
            return false;
        }
        String eL = "(((01[0-9]{2}|0[2-9][0-9]{2}|[1-9][0-9]{3})-(0?[13578]|1[02])-" + "(0?[1-9]|[12]\\\\d|3[01]))|((01[0-9]{2}|0[2-9][0-9]{2}|[1-9][0-9]{3})-" + "(0?[13456789]|1[012])-(0?[1-9]|[12]\\\\d|30))|((01[0-9]{2}|0[2-9][0-9]{2}|[1-9][0-9]{3})-0?2-" + "(0?[1-9]|1\\\\d|2[0-8]))|(((1[6-9]|[2-9]\\\\d)(0[48]|[2468][048]|[13579][26])|((04|08|12|16|[2468][048]|" + "[3579][26])00))-0?2-29)) (20|21|22|23|[0-1]?\\\\d):[0-5]?\\\\d:[0-5]?\\\\d";
        Pattern pat = Pattern.compile(eL);
        Matcher matcher = pat.matcher(sDateTime);
        return matcher.matches();
    }

}
